==> 0.6.4.tar.gz: lib/src/view/object_editor.dart <==
part of fields.view;

final Tag objectEditor = createTag((_) => new ObjectEditor());

class ObjectEditorResources extends Resources {
  String get style =>
"""
@host {
  * {
    background-color: ${backgroundColor};
    border:1px solid ${borderColor};
    height:100%;    
    overflow:hidden;
    width:100%;
  }
}
button, input {
  background-color:transparent;
  border-style:none;
  color:${textColor};
}
input[type="text"] {  
  width:100%;
}
button:disabled, input[type="text"][disabled] { 
  color:${textDisabledColor};
}
input[class=error] {
  border: 2px solid ${errorColor};
}
select {
  background-color:transparent;
  border: 0;
  border-radius: 0;
  color:${textColor};
}
option {
  background: ${fieldBackgroundColor};
}
table {
  border-collapse:collapse;
  border-top:1px solid ${borderColor};
  border-left:1px solid ${borderColor};
  margin:-1px;            
  padding:0px;
  table-layout:fixed;
  width:100%;
}
td {
  background-color: ${fieldBackgroundColor};
  border-top:1px solid ${borderColor};
  border-left:1px solid ${borderColor};
  color: ${textColor};
  vertical-align:text-top;
  margin:0px;
  overflow:hidden;
  padding:0px;
  text-overflow:ellipsis;
}
.expander {
  background-color:transparent;
  border-style:none;
  color:${textColor};
  float:left;   
  width:16px;
  padding:0;
  margin-top:4px;
}
.editorbox {
  display:block; 
  overflow:hidden;  
  text-overflow:ellipsis;  
  white-space:nowrap;
}
.filterbox {  
  display:block; 
  overflow:hidden;
}
.filter {
  background-color:transparent;
  border-style:none;
  color:${textColor};
  width:100%;
}
.namebox {
  padding-left:4px;
  width:20%;
}
.toggle {
  background-color:transparent;
  border-style:none;
  color:${textColor};
  float:left;
}
.toggle:disabled {
  opacity:0.4;
}
""";
  final String backgroundColor      = 'transparent';
  final String borderColor          = 'grey';
  final String errorColor           = 'red';
  final String fieldBackgroundColor = 'transparent';
  final String textColor            = 'black';
  final String textDisabledColor    = 'grey';
  const ObjectEditorResources();
}

class ObjectEditorDarkResources extends ObjectEditorResources {
  final String backgroundColor      = '#2d2d30';
  final String borderColor          = '#2d2d30';
  final String fieldBackgroundColor = '#252526';
  final String textColor            = '#f1f1f1';
  const ObjectEditorDarkResources();
}

class ObjectEditor extends HtmlElement with Themeable {
  
  static const TAG = 'object-editor';
  static ObjectEditorResources _factoryResources;

  String _filterText = '';  
  // TODO(rms): investigate the runtime error we get here with dart2js
//  Predicate _filter;
  Predicate _filterPrivate;
    
  final ObjectEditorResources resources;
  final StyleElement styleElement = new StyleElement();
    
  ButtonElement _sortButton;
  ButtonElement _lockButton;
  InputElement _filterBox;
  
  FieldList _list;
  FieldListView _listView;
  StreamSubscription _listListener;
  
  /// Returns `true` if the [selected] object is not `null`.
  bool get hasSelected => _selected != null;

  Object _selected;
  /// The current object selected for edition; may be `null`.
  Object 
    get selected => _selected;
    set selected(Object value) {           
      if(value != selected) {
        if (_listListener != null) _listListener.cancel();      
        _selected = value;
        _list = new FieldList()
        ..refreshSource(_selected)
        ..predicate = _filterPrivate;      
        _listListener = _list.fieldChanged.listen((_) {
          _selectedChanged.add(_selected);
        });
        _listView.clearCache();
        _refresh();
        _selectedChanged.add(_selected);
      }
    }
  
  final _selectedChanged = new StreamController<Object>.broadcast();
  Stream<Object> get selectedChanged => _selectedChanged.stream;
  
  /// Creates a new object editor with the given [resources].
  factory ObjectEditor(
      [ObjectEditorResources resources = const ObjectEditorResources()]) {    
    _factoryResources = resources;
    return new Element.tag(TAG);
  }
  
  ObjectEditor.created() 
  : super.created()
  , this.resources = _factoryResources {
    
    final _filter = (Field f) => f.name.contains(_filterText);
    _filterPrivate = (Field f) => !f.isPrivate && _filter(f);
    
    children(this, () {
      div(() {
        shadow(() {
          append(styleElement)..text = resources.style;
          _sortButton = toggleButton(() {
            span()..innerHtml = '&#9650;'..classes = ['up'];
            span()..innerHtml = '&#9660;'..classes = ['down'];
          })
          ..classes = ['toggle']
          ..disabled = true
          ..onToggle.listen((bool isDown) {
            if (isDown) {
              _list.comparator = 
                  (Field a, Field b) => a.name.compareTo(b.name) * -1;
            } else {
              _list.comparator = FieldList.defaultComparator;
            }
            _refresh();
          });
          _lockButton = toggleButton(() {
            span()..text = 'unlock'..classes = ['up'];
            span()..text = 'lock'..classes = ['down'];
          })
          ..classes = ['toggle']
          ..disabled = true
          ..onToggle.listen((bool isDown) {
            if (isDown) {
              _list.predicate = _filter;
            } else {
              _list.predicate = _filterPrivate;
            }
            _refresh();
          });
          span(() {
            _filterBox = input()
            ..type = 'text'
            ..classes = ['filter']
            ..disabled = true
            ..placeholder = 'filter fields';
          })
          ..classes = ['filterbox']
          ..onInput.listen((_) {
            _filterText = _filterBox.value;
            _list.refreshView();
            _refresh();
          });
          _listView = append(new FieldListView());
        });
      });
    });
    
    registerDefaultValueTransforms();    
    registerDefaultFieldEditors();
  }
  
  /// Refreshes this object editor to reflect the current state of the
  /// [selected] object.
  /// 
  /// This method does _not_ need to be called for any changes made to the 
  /// [selected] object through this object editor or any field editors.  It 
  /// _does_ need to be called when the [selected] object is modified external
  /// to this library, for example by another user interface element.
  void refresh() {
    if (hasSelected) {
      _list.refreshSource(_selected);
      _refresh();
    }
  }
  
  void _refresh() { 
    _sortButton.disabled = !hasSelected;
    _lockButton.disabled = !hasSelected;
    _filterBox.disabled = !hasSelected;
    _listView.refresh(_list);
  }
}

==> 0.6.5.tar.gz: lib/src/view/object_editor.dart <==
part of fields.view;

final Tag objectEditor = createTag((_) => new ObjectEditor());

class ObjectEditorResources extends Resources {
  String get style =>
"""
:host {
  background-color: ${backgroundColor};
  border:1px solid ${borderColor};
  height:100%;    
  overflow:hidden;
  width:100%;
}
.host {
  background-color: ${backgroundColor};
  border:1px solid ${borderColor};
  height:100%;    
  overflow:hidden;
  width:100%;
}
button, input {
  background-color:transparent;
  border-style:none;
  color:${textColor};
}
input[type="text"] {  
  width:100%;
}
button:disabled, input[type="text"][disabled] { 
  color:${textDisabledColor};
}
input[class=error] {
  border: 2px solid ${errorColor};
}
select {
  background-color:transparent;
  border: 0;
  border-radius: 0;
  color:${textColor};
}
option {
  background: ${fieldBackgroundColor};
}
table {
  border-collapse:collapse;
  border-top:1px solid ${borderColor};
  border-left:1px solid ${borderColor};
  margin:-1px;            
  padding:0px;
  table-layout:fixed;
  width:100%;
}
td {
  background-color: ${fieldBackgroundColor};
  border-top:1px solid ${borderColor};
  border-left:1px solid ${borderColor};
  color: ${textColor};
  vertical-align:text-top;
  margin:0px;
  overflow:hidden;
  padding:0px;
  text-overflow:ellipsis;
}
.expander {
  background-color:transparent;
  border-style:none;
  color:${textColor};
  float:left;   
  width:16px;
  padding:0;
  margin-top:4px;
}
.editorbox {
  display:block; 
  overflow:hidden;  
  text-overflow:ellipsis;  
  white-space:nowrap;
}
.filterbox {  
  display:block; 
  overflow:hidden;
}
.filter {
  background-color:transparent;
  border-style:none;
  color:${textColor};
  width:100%;
}
.namebox {
  padding-left:4px;
  width:20%;
}
.toggle {
  background-color:transparent;
  border-style:none;
  color:${textColor};
  float:left;
}
.toggle:disabled {
  opacity:0.4;
}
""";
  final String backgroundColor      = 'transparent';
  final String borderColor          = 'grey';
  final String errorColor           = 'red';
  final String fieldBackgroundColor = 'transparent';
  final String textColor            = 'black';
  final String textDisabledColor    = 'grey';
  const ObjectEditorResources();
}

class ObjectEditorDarkResources extends ObjectEditorResources {
  final String backgroundColor      = '#2d2d30';
  final String borderColor          = '#2d2d30';
  final String fieldBackgroundColor = '#252526';
  final String textColor            = '#f1f1f1';
  const ObjectEditorDarkResources();
}

class ObjectEditor extends HtmlElement with Themeable {
  
  static const TAG = 'object-editor';
  static ObjectEditorResources _factoryResources;

  String _filterText = '';  
  Predicate _filter;
  Predicate _filterPrivate;
    
  final ObjectEditorResources resources;
  final StyleElement styleElement = new StyleElement();
    
  ButtonElement _sortButton;
  ButtonElement _lockButton;
  InputElement _filterBox;
  
  FieldList _list;
  FieldListView _listView;
  StreamSubscription _listListener;
  
  /// Returns `true` if the [selected] object is not `null`.
  bool get hasSelected => _selected != null;

  Object _selected;
  /// The current object selected for edition; may be `null`.
  Object 
    get selected => _selected;
    set selected(Object value) {           
      if(value != selected) {
        if (_listListener != null) _listListener.cancel();      
        _selected = value;
        _list = new FieldList()
        ..refreshSource(_selected)
        ..predicate = _filterPrivate;      
        _listListener = _list.fieldChanged.listen((_) {
          _selectedChanged.add(_selected);
        });
        _listView.clearCache();
        _refresh();
        _selectedChanged.add(_selected);
      }
    }
  
  final _selectedChanged = new StreamController<Object>.broadcast();
  Stream<Object> get selectedChanged => _selectedChanged.stream;
  
  /// Creates a new object editor with the given [resources].
  factory ObjectEditor(
      [ObjectEditorResources resources = const ObjectEditorResources()]) {    
    _factoryResources = resources;
    return new Element.tag(TAG);
  }
  
  ObjectEditor.created() 
  : super.created()
  , this.resources = _factoryResources {
    
    _filter = (Field f) => f.name.contains(_filterText);
    _filterPrivate = (Field f) => !f.isPrivate && _filter(f);
    
    children(this, () {
      div(() {
        // TODO(rms): in Chrome 33 :host is behind an experimental flag
        //shadow(() {
          append(styleElement)..text = resources.style;
          _sortButton = toggleButton(() {
            span()..innerHtml = '&#9650;'..classes = ['up'];
            span()..innerHtml = '&#9660;'..classes = ['down'];
          })
          ..classes = ['toggle']
          ..disabled = true
          ..onToggle.listen((bool isDown) {
            if (isDown) {
              _list.comparator = 
                  (Field a, Field b) => a.name.compareTo(b.name) * -1;
            } else {
              _list.comparator = FieldList.defaultComparator;
            }
            _refresh();
          });
          _lockButton = toggleButton(() {
            span()..text = 'unlock'..classes = ['up'];
            span()..text = 'lock'..classes = ['down'];
          })
          ..classes = ['toggle']
          ..disabled = true
          ..onToggle.listen((bool isDown) {
            if (isDown) {
              _list.predicate = _filter;
            } else {
              _list.predicate = _filterPrivate;
            }
            _refresh();
          });
          span(() {
            _filterBox = input()
            ..type = 'text'
            ..classes = ['filter']
            ..disabled = true
            ..placeholder = 'filter fields';
          })
          ..classes = ['filterbox']
          ..onInput.listen((_) {
            _filterText = _filterBox.value;
            _list.refreshView();
            _refresh();
          });
          _listView = append(new FieldListView());
        //});
      })..classes = ['host'];
    });
    
    registerDefaultValueTransforms();    
    registerDefaultFieldEditors();
  }
  
  /// Refreshes this object editor to reflect the current state of the
  /// [selected] object.
  /// 
  /// This method does _not_ need to be called for any changes made to the 
  /// [selected] object through this object editor or any field editors.  It 
  /// _does_ need to be called when the [selected] object is modified external
  /// to this library, for example by another user interface element.
  void refresh() {
    if (hasSelected) {
      _list.refreshSource(_selected);
      _refresh();
    }
  }
  
  void _refresh() { 
    _sortButton.disabled = !hasSelected;
    _lockButton.disabled = !hasSelected;
    _filterBox.disabled = !hasSelected;
    _listView.refresh(_list);
  }
}
